@@ -19,21 +19,24 @@ namespace Seld\JsonLint;
 class Lexer
 {
     private $EOF = 1;
+    /**
+     * @phpstan-var array<int, string>
+     */
     private $rules = array(
-        0 => '/^\s+/',
-        1 => '/^-?([0-9]|[1-9][0-9]+)(\.[0-9]+)?([eE][+-]?[0-9]+)?\b/',
-        2 => '{^"(?>\\\\["bfnrt/\\\\]|\\\\u[a-fA-F0-9]{4}|[^\0-\x1f\\\\"]++)*+"}',
-        3 => '/^\{/',
-        4 => '/^\}/',
-        5 => '/^\[/',
-        6 => '/^\]/',
-        7 => '/^,/',
-        8 => '/^:/',
-        9 => '/^true\b/',
-        10 => '/^false\b/',
-        11 => '/^null\b/',
-        12 => '/^$/',
-        13 => '/^./',
+        0 => '/\G\s+/',
+        1 => '/\G-?([0-9]|[1-9][0-9]+)(\.[0-9]+)?([eE][+-]?[0-9]+)?\b/',
+        2 => '{\G"(?>\\\\["bfnrt/\\\\]|\\\\u[a-fA-F0-9]{4}|[^\0-\x1f\\\\"]++)*+"}',
+        3 => '/\G\{/',
+        4 => '/\G\}/',
+        5 => '/\G\[/',
+        6 => '/\G\]/',
+        7 => '/\G,/',
+        8 => '/\G:/',
+        9 => '/\Gtrue\b/',
+        10 => '/\Gfalse\b/',
+        11 => '/\Gnull\b/',
+        12 => '/\G$/',
+        13 => '/\G./',
     );
 
     private $conditions = array(
@@ -47,7 +50,7 @@ class Lexer
     private $input;
     private $more;
     private $done;
-    private $matched;
+    private $offset;
 
     public $match;
     public $yylineno;
@@ -70,8 +73,9 @@ class Lexer
         $this->input = $input;
         $this->more = false;
         $this->done = false;
+        $this->offset = 0;
         $this->yylineno = $this->yyleng = 0;
-        $this->yytext = $this->matched = $this->match = '';
+        $this->yytext = $this->match = '';
         $this->conditionStack = array('INITIAL');
         $this->yylloc = array('first_line' => 1, 'first_column' => 0, 'last_line' => 1, 'last_column' => 0);
 
@@ -81,26 +85,40 @@ class Lexer
     public function showPosition()
     {
         $pre = str_replace("\n", '', $this->getPastInput());
-        $c = str_repeat('-', max(0, strlen($pre) - 1)); // new Array(pre.length + 1).join("-");
+        $c = str_repeat('-', max(0, \strlen($pre) - 1)); // new Array(pre.length + 1).join("-");
 
         return $pre . str_replace("\n", '', $this->getUpcomingInput()) . "\n" . $c . "^";
     }
 
     public function getPastInput()
     {
-        $past = substr($this->matched, 0, strlen($this->matched) - strlen($this->match));
+        $pastLength = $this->offset - \strlen($this->match);
 
-        return (strlen($past) > 20 ? '...' : '') . substr($past, -20);
+        return ($pastLength > 20 ? '...' : '') . substr($this->input, max(0, $pastLength - 20), min(20, $pastLength));
     }
 
     public function getUpcomingInput()
     {
         $next = $this->match;
-        if (strlen($next) < 20) {
-            $next .= substr($this->input, 0, 20 - strlen($next));
+        if (\strlen($next) < 20) {
+            $next .= substr($this->input, $this->offset, 20 - \strlen($next));
         }
 
-        return substr($next, 0, 20) . (strlen($next) > 20 ? '...' : '');
+        return substr($next, 0, 20) . (\strlen($next) > 20 ? '...' : '');
+    }
+
+    public function getFullUpcomingInput()
+    {
+        $next = $this->match;
+        if (substr($next, 0, 1) === '"' && substr_count($next, '"') === 1) {
+            $len = \strlen($this->input);
+            $strEnd = min(strpos($this->input, '"', $this->offset + 1) ?: $len, strpos($this->input, "\n", $this->offset + 1) ?: $len);
+            $next .= substr($this->input, $this->offset, $strEnd - $this->offset);
+        } elseif (\strlen($next) < 20) {
+            $next .= substr($this->input, $this->offset, 20 - \strlen($next));
+        }
+
+        return $next;
     }
 
     protected function parseError($str, $hash)
@@ -113,7 +131,7 @@ class Lexer
         if ($this->done) {
             return $this->EOF;
         }
-        if ($this->input === '') {
+        if ($this->offset === \strlen($this->input)) {
             $this->done = true;
         }
 
@@ -128,29 +146,28 @@ class Lexer
         }
 
         $rules = $this->getCurrentRules();
-        $rulesLen = count($rules);
+        $rulesLen = \count($rules);
 
         for ($i=0; $i < $rulesLen; $i++) {
-            if (preg_match($this->rules[$rules[$i]], $this->input, $match)) {
+            if (preg_match($this->rules[$rules[$i]], $this->input, $match, 0, $this->offset)) {
                 preg_match_all('/\n.*/', $match[0], $lines);
                 $lines = $lines[0];
                 if ($lines) {
-                    $this->yylineno += count($lines);
+                    $this->yylineno += \count($lines);
                 }
 
                 $this->yylloc = array(
                     'first_line' => $this->yylloc['last_line'],
                     'last_line' => $this->yylineno+1,
                     'first_column' => $this->yylloc['last_column'],
-                    'last_column' => $lines ? strlen($lines[count($lines) - 1]) - 1 : $this->yylloc['last_column'] + strlen($match[0]),
+                    'last_column' => $lines ? \strlen($lines[\count($lines) - 1]) - 1 : $this->yylloc['last_column'] + \strlen($match[0]),
                 );
                 $this->yytext .= $match[0];
                 $this->match .= $match[0];
-                $this->yyleng = strlen($this->yytext);
+                $this->yyleng = \strlen($this->yytext);
                 $this->more = false;
-                $this->input = substr($this->input, strlen($match[0]));
-                $this->matched .= $match[0];
-                $token = $this->performAction($rules[$i], $this->conditionStack[count($this->conditionStack)-1]);
+                $this->offset += \strlen($match[0]);
+                $token = $this->performAction($rules[$i], $this->conditionStack[\count($this->conditionStack)-1]);
                 if ($token) {
                     return $token;
                 }
@@ -159,7 +176,7 @@ class Lexer
             }
         }
 
-        if ($this->input === "") {
+        if ($this->offset === \strlen($this->input)) {
             return $this->EOF;
         }
 
@@ -175,7 +192,7 @@ class Lexer
 
     private function getCurrentRules()
     {
-        return $this->conditions[$this->conditionStack[count($this->conditionStack)-1]]['rules'];
+        return $this->conditions[$this->conditionStack[\count($this->conditionStack)-1]]['rules'];
     }
 
     private function performAction($avoiding_name_collisions, $YY_START)
@@ -185,7 +202,6 @@ class Lexer
             break;
         case 1:
             return 6;
-           break;
         case 2:
             $this->yytext = substr($this->yytext, 1, $this->yyleng-2);
 
